package com.ruoyi.bst.storage.service.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.github.pagehelper.PageHelper;
import com.ruoyi.bst.location.domain.LocationQuery;
import com.ruoyi.bst.location.domain.LocationVO;
import com.ruoyi.bst.storage.domain.dto.DepositStorageDTO;
import com.ruoyi.bst.storage.domain.dto.TakeOutWineDTO;
import com.ruoyi.bst.storage.domain.enums.StorageStatus;
import com.ruoyi.bst.storageRecord.domain.StorageRecord;
import com.ruoyi.bst.storageRecord.service.StorageRecordService;
import com.ruoyi.bst.storeStaff.domain.enums.StoreStaffPermission;
import com.ruoyi.common.core.domain.vo.UserVO;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.user.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.bst.storage.mapper.StorageMapper;
import com.ruoyi.bst.storage.domain.Storage;
import com.ruoyi.bst.storage.domain.StorageVO;
import com.ruoyi.bst.storage.domain.StorageQuery;
import com.ruoyi.bst.storage.service.StorageService;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * 存酒Service业务层处理
 *
 * @author ruoyi
 * @date 2025-06-06
 */
@Service
public class StorageServiceImpl implements StorageService {
    @Autowired
    private StorageMapper storageMapper;
    @Autowired
    private TransactionTemplate transactionTemplate;
    @Autowired
    private StorageRecordService storageRecordService;
    @Autowired
    private StorageService storageService;
    @Autowired
    private UserService userService;

    /**
     * 查询存酒
     *
     * @param id 存酒主键
     * @return 存酒
     */
    @Override
    public StorageVO selectStorageById(Long id, boolean scope) {
        if (id == null) {
            return null;
        }
        StorageQuery query = new StorageQuery();
        query.setId(id);
        query.setScope(scope);
        query.addStorePermission(StoreStaffPermission.STORAGE_VIEW.getCode());
        return this.selectOne(query);
    }


    @Override
    public StorageVO selectOne(StorageQuery query) {
        PageHelper.startPage(1, 1);
        List<StorageVO> list = this.selectStorageList(query);
        return CollectionUtils.firstElement(list);
    }

    /**
     * 查询存酒列表
     *
     * @param storage 存酒
     * @return 存酒
     */
    @Override
    public List<StorageVO> selectStorageList(StorageQuery storage) {
        return storageMapper.selectStorageList(storage);
    }

    /**
     * 新增存酒
     *
     * @param storage 存酒
     * @return 结果
     */
    @Override
    public int insertStorage(Storage storage) {
        storage.setCreateTime(DateUtils.getNowDate());
        return storageMapper.insertStorage(storage);
    }

    /**
     * 修改存酒
     *
     * @param storage 存酒
     * @return 结果
     */
    @Override
    public int updateStorage(Storage storage) {
        return storageMapper.updateStorage(storage);
    }

    /**
     * 批量删除存酒
     *
     * @param ids 需要删除的存酒主键
     * @return 结果
     */
    @Override
    public int deleteStorageByIds(Long[] ids) {
        return storageMapper.deleteStorageByIds(ids);
    }

    /**
     * 删除存酒信息
     *
     * @param id 存酒主键
     * @return 结果
     */
    @Override
    public int deleteStorageById(Long id) {
        return storageMapper.deleteStorageById(id);
    }

    @Override
    public int storageWine(DepositStorageDTO dto) {
        // 新建集合用于数据插入
        List<Storage> list = new ArrayList<>();

        // 根据电话查询用户信息
        UserVO user = userService.selectUserByPhonenumber(dto.getPhone());
        ServiceUtil.assertion(user == null,"当前用户不存在");

        // 设置对应属性进行批量插入
        dto.getStorage().forEach(storage -> {
            storage.setRemark(dto.getRemark());
            storage.setCreateTime(DateUtils.getNowDate());
            storage.setUserId(user.getUserId());
            list.add(storage);
        });
        return storageMapper.batchInsert(list);
    }

    @Override
    public List<StorageVO> selectCurrentNum(StorageQuery query) {
        return storageMapper.selectCurrentNum(query);
    }


    @Override
    public int getWine(List<TakeOutWineDTO> list) {
        // 查询需要更新的存酒信息id列表
        List<Long> ids = list.stream()
                .map(TakeOutWineDTO::getStorageId)
                .collect(Collectors.toList());

            // 查询出对应的map用于更新数据
            StorageQuery query = new StorageQuery();
            query.setStorageIds(ids);
            Map<Long, StorageVO> storageMap = storageMapper.selectStorageList(query)
                    .stream()
                    .collect(Collectors.toMap(Storage::getId, StorageVO -> StorageVO));

            List<TakeOutWineDTO> toStorageUpdate = new ArrayList<>();
            List<StorageRecord> toStorageRecordInsert = new ArrayList<>();

            // 生成数据
            for (TakeOutWineDTO dto : list) {
                StorageVO storageVO = storageMap.get(dto.getStorageId());
                if (storageVO == null) {
                    continue;
                }

                // 生成数据用于批量取酒
                TakeOutWineDTO data = new TakeOutWineDTO();
                data.setStorageId(storageVO.getId());
                data.setNumber(dto.getNumber());
                toStorageUpdate.add(data);

                // 生成记录
                StorageRecord record = new StorageRecord();
                record.setStorageId(storageVO.getId());
                record.setStoreId(storageVO.getStoreId());
                record.setUserId(storageVO.getUserId());
                record.setReason("用户" + storageVO.getUserName() + "取酒" + dto.getNumber() + "瓶");
                record.setNumber(dto.getNumber());
                record.setCreateTime(DateUtils.getNowDate());
                record.setBeforeNum(storageVO.getCurrentNum());
                record.setAfterNum(storageVO.getCurrentNum()-dto.getNumber());
                toStorageRecordInsert.add(record);
            }

        Integer result = transactionTemplate.execute(status -> {
            // 批量减少数量,根据ID限制
            int i = storageMapper.batchGetWine(toStorageUpdate);
            ServiceUtil.assertion(i != toStorageUpdate.size(), "物品数量不足,取酒失败");

            // 更新已取完的数据状态
            StorageQuery storageQuery = new StorageQuery();
            storageQuery.setStorageIds(toStorageUpdate.stream().map(TakeOutWineDTO::getStorageId).collect(Collectors.toList()));
            storageQuery.setTakenAll(true);
            List<StorageVO> storageList = storageMapper.selectStorageList(storageQuery);
            storageList.forEach(storage -> {
                storage.setStatus(StorageStatus.COMPLETED.getCode());
            });
            i = storageMapper.batchUpdate(storageList);
            ServiceUtil.assertion(i != storageList.size(), "更新存放状态失败");

            // 批量插入记录
            i = storageRecordService.batchInsert(toStorageRecordInsert);
            ServiceUtil.assertion(i <= 0, "生成记录失败");
            return i;
        });
        return result == null ? 0 : result;
    }
}
